@@ -27,8 +27,6 @@ extern "C" {
 #endif
 #endif
 
-#undef ENABLE_ENHANCED
-
 typedef enum {
   PINENTRY_COLOR_NONE, PINENTRY_COLOR_DEFAULT,
   PINENTRY_COLOR_BLACK, PINENTRY_COLOR_RED,
@@ -39,20 +37,28 @@ typedef enum {
 
 struct pinentry
 {
-  /* The window title, or NULL.  */
+  /* The window title, or NULL.  (Assuan: "SETTITLE TITLE".)  */
   char *title;
-  /* The description to display, or NULL.  */
+  /* The description to display, or NULL.  (Assuan: "SETDESC
+     DESC".) */
   char *description;
-  /* The error message to display, or NULL.  */
+  /* The error message to display, or NULL.  (Assuan: "SETERROR
+     MESSAGE".) */
   char *error;
-  /* The prompt to display, or NULL.  */
+  /* The prompt to display, or NULL.  (Assuan: "SETPROMPT
+     prompt".)  */
   char *prompt;
-  /* The OK button text to display, or NULL.  */
+  /* The OK button text to display, or NULL.  (Assuan: "SETOK
+     OK".)  */
   char *ok;
-  /* The Not-OK button text to display, or NULL.  */
+  /* The Not-OK button text to display, or NULL.  This is the text for
+     the alternative option shown by the third button.  (Assuan:
+     "SETNOTOK NOTOK".)  */
   char *notok;
-  /* The Cancel button text to display, or NULL.  */
+  /* The Cancel button text to display, or NULL.  (Assuan: "SETCANCEL
+     CANCEL".)  */
   char *cancel;
+
   /* The buffer to store the secret into.  */
   char *pin;
   /* The length of the buffer.  */
@@ -61,15 +67,19 @@ struct pinentry
      the user (0). */
   int pin_from_cache;
 
-  /* The name of the X display to use if X is available and supported.  */
+  /* The name of the X display to use if X is available and supported.
+     (Assuan: "OPTION display DISPLAY".)  */
   char *display;
-  /* The name of the terminal node to open if X not available or supported.  */
+  /* The name of the terminal node to open if X not available or
+     supported.  (Assuan: "OPTION ttyname TTYNAME".)  */
   char *ttyname;
-  /* The type of the terminal.  */
+  /* The type of the terminal.  (Assuan: "OPTION ttytype TTYTYPE".)  */
   char *ttytype;
-  /* The LC_CTYPE value for the terminal.  */
+  /* The LC_CTYPE value for the terminal.  (Assuan: "OPTION lc-ctype
+     LC_CTYPE".)  */
   char *lc_ctype;
-  /* The LC_MESSAGES value for the terminal.  */
+  /* The LC_MESSAGES value for the terminal.  (Assuan: "OPTION
+     lc-messages LC_MESSAGES".)  */
   char *lc_messages;
 
   /* True if debug mode is requested.  */
@@ -78,40 +88,38 @@ struct pinentry
   /* The number of seconds before giving up while waiting for user input. */
   int timeout;
 
-#ifdef ENABLE_ENHANCED
-  /* True if enhanced mode is requested.  */
-  int enhanced;
-#endif
-
-  /* True if caller should grab the keyboard.  */
+  /* True if caller should grab the keyboard.  (Assuan: "OPTION grab"
+     or "OPTION no-grab".)  */
   int grab;
   /* The window ID of the parent window over which the pinentry window
-     should be displayed.  */
+     should be displayed.  (Assuan: "OPTION parent-wid WID".)  */
   int parent_wid;
 
   /* The name of an optional file which will be touched after a curses
-     entry has been displayed.  */
+     entry has been displayed.  (Assuan: "OPTION touch-file
+     FILENAME".)  */
   char *touch_file;
 
-  /* The user should set this to -1 if the user canceled the request,
-     and to the length of the PIN stored in pin otherwise.  */
+  /* The frontend should set this to -1 if the user canceled the
+     request, and to the length of the PIN stored in pin
+     otherwise.  */
   int result;
 
-  /* The user should set this if the NOTOK button was pressed.  */
+  /* The frontend should set this if the NOTOK button was pressed.  */
   int canceled;
 
-  /* The user should set this to true if an error with the local
+  /* The frontend should set this to true if an error with the local
      conversion occured. */
   int locale_err;
 
-  /* The user should set this to an gpg-error so that commands are
-     abale to return specific error codes.  This is an ugly hack due
-     to the fact that pinentry_cmd_handler_t return the length of the
-     passphrase or an negative error code.  */
+  /* The frontend should set this to a gpg-error so that commands are
+     able to return specific error codes.  This is an ugly hack due to
+     the fact that pinentry_cmd_handler_t returns the length of the
+     passphrase or a negative error code.  */
   int specific_err;
 
-  /* The user should set this to true if the window close button has
-     been used.  This flag is used in addition to a regular return
+  /* The frontend should set this to true if the window close button
+     has been used.  This flag is used in addition to a regular return
      value.  */
   int close_button;
 
@@ -122,10 +130,11 @@ struct pinentry
 
   /* If true a second prompt for the passphrase is shown and the user
      is expected to enter the same passphrase again.  Pinentry checks
-     that both match.  */
+     that both match.  (Assuan: "SETREPEAT".)  */
   char *repeat_passphrase;
 
-  /* The string to show if a repeated passphrase does not match.  */
+  /* The string to show if a repeated passphrase does not match.
+     (Assuan: "SETREPEATERROR ERROR".)  */
   char *repeat_error_string;
 
   /* Set to true if the passphrase has been entered a second time and
@@ -135,10 +144,12 @@ struct pinentry
   /* If this is not NULL, a passphrase quality indicator is shown.
      There will also be an inquiry back to the caller to get an
      indication of the quality for the passphrase entered so far.  The
-     string is used as a label for the quality bar.  */
+     string is used as a label for the quality bar.  (Assuan:
+     "SETQUALITYBAR LABEL".)  */
   char *quality_bar;
 
-  /* The tooltip to be show for the qualitybar.  Malloced or NULL.  */
+  /* The tooltip to be show for the qualitybar.  Malloced or NULL.
+     (Assuan: "SETQUALITYBAR_TT TOOLTIP".)  */
   char *quality_bar_tt;
 
   /* For the curses pinentry, the color of error messages.  */
@@ -151,19 +162,25 @@ struct pinentry
   /* Malloced and i18ned default strings or NULL.  These strings may
      include an underscore character to indicate an accelerator key.
      A double underscore represents a plain one.  */
+  /* (Assuan: "OPTION default-ok OK").  */
   char *default_ok;
+  /* (Assuan: "OPTION default-cancel CANCEL").  */
   char *default_cancel;
+  /* (Assuan: "OPTION default-prompt PROMPT").  */
   char *default_prompt;
+  /* (Assuan: "OPTION default-pwmngr
+     SAVE_PASSWORD_WITH_PASSWORD_MANAGER?").  */
   char *default_pwmngr;
 
   /* Whether we are allowed to read the password from an external
-     cache.  */
+     cache.  (Assuan: "OPTION allow-external-password-cache")  */
   int allow_external_password_cache;
 
   /* We only try the cache once.  */
   int tried_password_cache;
 
-  /* A stable identifier for the key.  */
+  /* A stable identifier for the key.  (Assuan: "SETKEYINFO
+     KEYINFO".)  */
   char *keyinfo;
 
   /* Whether we may cache the password (according to the user).  */
@@ -185,7 +202,7 @@ typedef struct pinentry *pinentry_t;
    entry.  On confirmation, the function should return TRUE if
    confirmed, and FALSE otherwise.  On PIN entry, the function should
    return -1 if an error occured or the user cancelled the operation
-   and the length of the secret otherwise.  */
+   and 1 otherwise.  */
 typedef int (*pinentry_cmd_handler_t) (pinentry_t pin);
 
 /* Start the pinentry event loop.  The program will start to process
@@ -216,6 +233,12 @@ int pinentry_inq_quality (pinentry_t pin,
    PIN.  Returns new buffer on success and 0 on failure.  */
 char *pinentry_setbufferlen (pinentry_t pin, int len);
 
+/* Use the buffer at BUFFER for PIN->PIN.  BUFFER must be NULL or
+   allocated using secmem_alloc.  LEN is the size of the buffer.  If
+   it is unknown, but BUFFER is a NUL terminated string, you pass 0 to
+   just use strlen(buffer)+1.  */
+void pinentry_setbuffer_use (pinentry_t pin, char *buffer, int len);
+
 /* Initialize the secure memory subsystem, drop privileges and
    return.  Must be called early.  */
 void pinentry_init (const char *pgmname);
